@@ -1,12 +1,12 @@
 # -*- coding:utf-8 -*-
 
-
 import pylink
 import telnetlib
 import serial
 import ecdsa
 
 from . import bflb_version
+from . import bflb_interface_cklink
 from . import bflb_interface_jlink
 from . import bflb_interface_uart
 from . import bflb_interface_sdio
@@ -24,18 +24,8 @@ from . import bflb_utils
 from . import bflb_configobj
 from . import bflb_fdt
 
-try:
-    import changeconf as cgc
-    conf_sign = True
-except ImportError:
-    conf_sign = False
-    
-if conf_sign:
-    from . import bl602
-else:
-    from . import bl60x
-    from . import bl602
-    from . import bl702
-    from . import bl808
-    from . import bl616
-    from . import wb03
+from . import bl602
+from . import bl702
+from . import bl702l
+from . import bl808
+from . import bl616
