@@ -1,6 +1,5 @@
 # -*- coding:utf-8 -*-
 
-
 import hashlib
 import binascii
 
@@ -41,8 +40,14 @@ rd_lock_key_slot_5 = 31
 
 
 # update efuse info
-def img_update_efuse(cfg, sign, pk_hash, flash_encryp_type, flash_key, sec_eng_key_sel,
-                     sec_eng_key):
+def img_update_efuse(cfg,
+                     sign,
+                     pk_hash,
+                     flash_encryp_type,
+                     flash_key,
+                     sec_eng_key_sel,
+                     sec_eng_key,
+                     security=False):
     fp = open(cfg.get("Img_Cfg", "efuse_file"), 'rb')
     efuse_data = bytearray(fp.read()) + bytearray(0)
     fp.close()
@@ -114,6 +119,11 @@ def img_update_efuse(cfg, sign, pk_hash, flash_encryp_type, flash_key, sec_eng_k
     # set read write lock key
     efuse_data[124:128] = bflb_utils.int_to_4bytearray_l(rw_lock)
     efuse_mask_data[124:128] = bflb_utils.int_to_4bytearray_l(rw_lock)
+    if security is True:
+        bflb_utils.printf("Encrypt efuse data")
+        security_key, security_iv = bflb_utils.get_security_key()
+        efuse_data = img_create_encrypt_data(efuse_data, security_key, security_iv, 0)
+        efuse_data = bytearray(4096) + efuse_data
     fp = open(cfg.get("Img_Cfg", "efuse_file"), 'wb+')
     fp.write(efuse_data)
     fp.close()
@@ -271,14 +281,15 @@ def encrypt_loader_bin_do(file, sign, encrypt, createcfg):
             newval = oldval
             if encrypt != 0:
                 newval = (newval | (1 << encrypt_type_pos))
-                newval = (newval | (0 << key_sel_pos))
+                newval = (newval | (1 << key_sel_pos))
             if sign != 0:
                 newval = (newval | (1 << sign_pos))
                 data_tohash += load_helper_bin_body_encrypt
                 publickey_file = cfg.get("Img_Cfg", "publickey_file")
                 privatekey_file_uecc = cfg.get("Img_Cfg", "privatekey_file_uecc")
-                pk_data, pk_hash, signature = img_create_sign_data(data_tohash, privatekey_file_uecc,
-                    publickey_file)
+                pk_data, pk_hash, signature = img_create_sign_data(data_tohash,
+                                                                   privatekey_file_uecc,
+                                                                   publickey_file)
                 pk_data = pk_data + bflb_utils.get_crc32_bytearray(pk_data)
             data[offset:offset + 4] = bflb_utils.int_to_4bytearray_l(newval)
             load_helper_bin_header = data
@@ -286,16 +297,17 @@ def encrypt_loader_bin_do(file, sign, encrypt, createcfg):
                 pk_data + signature + aesiv_data + load_helper_bin_body_encrypt
             # calculate hash
             hashfun = hashlib.sha256()
-            hashfun.update(load_helper_bin_body_encrypt)
+            hashfun.update(data_tohash)
             hash = bflb_utils.hexstr_to_bytearray(hashfun.hexdigest())
             # update hash & crc
             load_helper_bin_data = bytearray(load_helper_bin_encrypt)
-            load_helper_bin_encrypt = img_create_update_bootheader_if(load_helper_bin_data, hash, 1)
+            load_helper_bin_encrypt = img_create_update_bootheader_if(
+                load_helper_bin_data, hash, 1)
         return True, load_helper_bin_encrypt
     return False, None
 
 
-def img_creat_process(flash_img, cfg):
+def img_creat_process(flash_img, cfg, security=False):
     encrypt_blk_size = 16
     padding = bytearray(encrypt_blk_size)
     data_tohash = bytearray(0)
@@ -419,19 +431,19 @@ def img_creat_process(flash_img, cfg):
                 # AES 128
                 img_update_efuse(cfg, sign, pk_hash, 1,
                                  encrypt_key + bytearray(32 - len(encrypt_key)), key_sel,
-                                 encrypt_key + bytearray(32 - len(encrypt_key)))
+                                 encrypt_key + bytearray(32 - len(encrypt_key)), security)
             if encrypt == 2:
                 # AES 256
                 img_update_efuse(cfg, sign, pk_hash, 3,
                                  encrypt_key + bytearray(32 - len(encrypt_key)), key_sel,
-                                 encrypt_key + bytearray(32 - len(encrypt_key)))
+                                 encrypt_key + bytearray(32 - len(encrypt_key)), security)
             if encrypt == 3:
                 # AES 192
                 img_update_efuse(cfg, sign, pk_hash, 2,
                                  encrypt_key + bytearray(32 - len(encrypt_key)), key_sel,
-                                 encrypt_key + bytearray(32 - len(encrypt_key)))
+                                 encrypt_key + bytearray(32 - len(encrypt_key)), security)
         else:
-            img_update_efuse(cfg, sign, pk_hash, encrypt, None, key_sel, None)
+            img_update_efuse(cfg, sign, pk_hash, encrypt, None, key_sel, None, security)
     else:
         bflb_utils.printf("Write if img")
         whole_img_file_name = cfg.get(cfg_section, "whole_img_file")
@@ -444,17 +456,17 @@ def img_creat_process(flash_img, cfg):
             if encrypt == 1:
                 # AES 128
                 img_update_efuse(cfg, sign, pk_hash, 1, None, key_sel,
-                                 encrypt_key + bytearray(32 - len(encrypt_key)))
+                                 encrypt_key + bytearray(32 - len(encrypt_key)), security)
             if encrypt == 2:
                 # AES 256
                 img_update_efuse(cfg, sign, pk_hash, 3, None, key_sel,
-                                 encrypt_key + bytearray(32 - len(encrypt_key)))
+                                 encrypt_key + bytearray(32 - len(encrypt_key)), security)
             if encrypt == 3:
                 # AES 192
                 img_update_efuse(cfg, sign, pk_hash, 2, None, key_sel,
-                                 encrypt_key + bytearray(32 - len(encrypt_key)))
+                                 encrypt_key + bytearray(32 - len(encrypt_key)), security)
         else:
-            img_update_efuse(cfg, sign, pk_hash, 0, None, key_sel, bytearray(32))
+            img_update_efuse(cfg, sign, pk_hash, 0, None, key_sel, bytearray(32), security)
     return "OK", data_tohash
 
 
@@ -467,12 +479,15 @@ def img_create_do(args, img_dir_path=None, config_file=None):
     cfg.read(config_file)
     img_type = "media"
     signer = "none"
+    security = False
     data_tohash = bytearray(0)
     try:
         if args.image:
             img_type = args.image
         if args.signer:
             signer = args.signer
+        if args.security:
+            security = (args.security == "efuse")
     except Exception as e:
         # bflb_utils.printf(help information and exit:)
         # will  something like "option -a not recognized")
@@ -484,13 +499,16 @@ def img_create_do(args, img_dir_path=None, config_file=None):
         flash_img = 0
 
     # deal image creation
-    ret, data_tohash = img_creat_process(flash_img, cfg)
+    ret, data_tohash = img_creat_process(flash_img, cfg, security)
     if ret != "OK":
         bflb_utils.printf("Fail to create images!")
+        return False
+    else:
+        return True
 
 
-def create_sp_media_image(config, cpu_type=None):
+def create_sp_media_image(config, cpu_type=None, security=False):
     bflb_utils.printf("========= sp image create =========")
     cfg = BFConfigParser()
     cfg.read(config)
-    img_creat_process(1, cfg)
+    img_creat_process(1, cfg, security)
